#include "nds.h"
#include <nds/arm9/console.h> //basic print funcionality
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <stdint.h>
#include <fat.h>

#include "pcap.h"
#include "aircrack-ptw-lib.h"

#define KEYLIMIT (1000000)
#define BSSIDLEN (6)

typedef struct {
	u_char bssid[BSSIDLEN];
	uint8_t keyindex;
	PTW_attackstate * state;
} network;

const uint8_t beginpacket[] = {0xAA, 0xAA, 0x03, 0x00, 0x00, 0x00, 0x08, 0x06, 0x00, 0x01, 0x08, 0x00, 0x06, 0x04, 0x00, 0x02 };
const uint8_t broadcast[] = {0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF};

void printKey(uint8_t * key, int keylen) {
	int i;
	printf("Key found, length %02d: ", keylen);
	for (i = 0; i < keylen; i++) {
		printf("%02X ", key[i]);
	}
	printf("\n");
}
//---------------------------------------------------------------------------------
int main(void) {
//---------------------------------------------------------------------------------

	//touchPosition touchXY;
	videoSetMode(0);	//not using the main screen
	videoSetModeSub(MODE_0_2D | DISPLAY_BG0_ACTIVE);	//sub bg 0 will be used to print text
	vramSetBankC(VRAM_C_SUB_BG);
	SUB_BG0_CR = BG_MAP_BASE(31);
	BG_PALETTE_SUB[255] = RGB15(31,31,31);	//by default font will be rendered with color 255
	consoleInitDefault((u16*)SCREEN_BASE_BLOCK_SUB(31), (u16*)CHAR_BASE_BLOCK_SUB(0), 16);
	fatInitDefault();

	//=================================================
	//char errbuf[PCAP_ERRBUF_SIZE];
	FILE * descr;
	u_char *packet;
	struct pcap_pkthdr hdr;
	struct pcap_file_header pfh;
	int bssoffset;
	int dstoffset;
	network * networktable = NULL;
	int currenttable;
	int numstates = 0;
	uint8_t key[PTW_KEYHSBYTES];
	uint8_t buf[PTW_n];
	uint8_t iv[3];
	uint8_t keystream[16];
	int k,i;
	int keyfound;

	printf("** AircrackDS **\n\n");

	descr = fopen("/ptw.cap", "rb");
	if (descr == NULL) {
		printf("no /ptw.cap");
		goto read_fail;
	}
	fread(&pfh,4,1,descr);
	if (pfh.magic != TCPDUMP_MAGIC && pfh.magic != TCPDUMP_CIGAM ) {
		printf("Not a cap file");
		goto no_pcap;
	}
	fread((uchar*)&pfh+4,20,1,descr);
	if( pfh.magic == TCPDUMP_CIGAM )
		SWAP32( pfh.linktype );
	if( pfh.linktype != LINKTYPE_IEEE802_11 && pfh.linktype != LINKTYPE_PRISM_HEADER && pfh.linktype != LINKTYPE_RADIOTAP_HDR ) {
			printf("This file is not a regular 802.11 (wireless) capture.\n" );
			goto no_wireless_cap;
	}

	printf("Parsing cap file...\n");
	packet = malloc (65535*sizeof(u_char));
	for(fread(&hdr,sizeof(hdr),1,descr);!feof(descr);fread(&hdr,sizeof(hdr),1,descr)) {
		if( pfh.magic == TCPDUMP_CIGAM )
			SWAP32( hdr.caplen );
		if( hdr.caplen <= 0 || hdr.caplen > 65535 ) {
			printf("\nInvalid packet capture length %d - corrupted file?\n", hdr.caplen );
			goto corrupted_cap;
		}
		fread(packet,hdr.caplen,1,descr);
		if (((hdr.len == 68) || (hdr.len == 86) ) && (packet[0] == 0x08)) {
			// ARP
			// Find the right state
			if (( packet[1] & 0x03) == 0x01) {
				bssoffset = 4;
				dstoffset = 10;
			} else {
				bssoffset = 10;
				dstoffset = 4;
			}
			currenttable = -1;
			for (k = 0; k < numstates; k++) {
				if ((memcmp(networktable[k].bssid, &packet[bssoffset], BSSIDLEN) == 0) && (networktable[k].keyindex == packet[27])) {
					currenttable = k;
				}
			}
			if (currenttable == -1) {
				// Allocate a new table
				printf("BSSID=%02X:%02X:%02X:%02X:%02X:%02X\nKey index=%01d\n", packet[bssoffset+0], packet[bssoffset+1], packet[bssoffset+2], packet[bssoffset+3], packet[bssoffset+4], packet[bssoffset+5], packet[27]);
				numstates++;
				networktable = realloc(networktable, numstates * sizeof(network));
				networktable[numstates-1].state = PTW_newattackstate();
				if(networktable[numstates-1].state == NULL) {
					printf("allocation problem !!!\n");
					exit(-1);
				}
				memcpy(networktable[numstates-1].bssid, &packet[bssoffset], BSSIDLEN);
				networktable[numstates-1].keyindex = packet[27];
				currenttable = numstates-1;
			}
			// Find IV
			for (i = 0; i < PTW_IVBYTES; i++) {
				iv[i] = packet[24+i];
			}
			for (i = 0; i < PTW_KSBYTES; i++) {
				keystream[i] = packet[28+i] ^ beginpacket[i];
			}
			if (memcmp(broadcast,&packet[dstoffset], 6) == 0) {
				// it is a request
				// printf("found request\n");
				keystream[PTW_KSBYTES-1] ^= 0x03;
			}
			for (i = 0; i < PTW_KSBYTES; i++) {
				buf[i] = keystream[i] ^ packet[28+i];
			}
			PTW_addsession(networktable[currenttable].state, iv, keystream);
		}
	}
	corrupted_cap:
	no_wireless_cap:
	no_pcap:
	fclose(descr);
	read_fail:
	printf("\nRecovering WEP KEY...\n");
	for ( k = 0; k < numstates; k++) {
		printf("For BSSID %02X:%02X:%02X:%02X:%02X:%02X\nKey Index=%01d packets=%d\nPlease be patient...\n",
		networktable[k].bssid[0],
		networktable[k].bssid[1],
		networktable[k].bssid[2],
		networktable[k].bssid[3],
		networktable[k].bssid[4],
		networktable[k].bssid[5],
		networktable[k].keyindex,
		networktable[k].state->packets_collected);
		keyfound=0;
		if(PTW_computeKey(networktable[k].state, key, 5, KEYLIMIT/10) == 1) {
			keyfound=1;
			printKey(key, 5);  
		}
		if(PTW_computeKey(networktable[k].state, key, 13, KEYLIMIT) == 1) {
			keyfound=1;
			printKey(key, 13);
		}
		if(keyfound==0) {
			printf("Sorry, key not found !");
		}
	}

	while(1) {

		//touchXY=touchReadXY();
		//iprintf("\x1b[10;0HTouch x = %04X, %04X\n", touchXY.x, touchXY.px);
		//iprintf("Touch y = %04X, %04X\n", touchXY.y, touchXY.py);

	}

	return 0;
}
